/**
   This is a simple abstract generic hash table. This
   abstract class leaves the technique of handling
   collisions up to an implementation. This abstract class
   also leaves the choice of the hash function up to each
   implementation.
*/
public abstract class HashTable<K,V>
{
   protected int length; // The length of the bucket array.
   protected int size;   // The number of items in the hash table.
   protected double maxLoadFactor;

   /**
      Each implementation should define an appropriate hash function.

      @param key  the item to plug into the hash function
      @return the hash function's hashed value for key
   */
   public abstract int hashFunction(K key);


   /**
      @return the number of key-value pairs stored in this hash table
   */
   public int size()
   {
      return size;
   }


   /**
      Return the hash table's current load factor which is
      the number of (key, value) pairs in the hash table
      divided by the number of hash buckets.

      @return this hash table's current load factor
   */
   public double loadFactor()
   {
      return ((double)size()) / length;
   }


   /**
      @param key    key that is hashed into a hash table index
      @param value  value that is placed in the hash table at the hashed index
   */
   public abstract void insert(K key, V value);


   /**
      Return a reference to the Value with the given Key.
      Return null if the Key is not found in the hash table.

      @param key  key that is hashed into a hash table index
      @return the value found at the hash table's hashed index
   */
   public abstract V search(K key);


   /**
      Remove the value with the given key.
      Return false if the key is not found in the hash table.

      @param key  key that is hashed into a hash table index
      @return true if the key was found and the (key,value) pair was deleted
   */
   public abstract boolean delete(K key);


   @Override public abstract String toString();
}
