/**
   An implementation of HashTable that uses String keys but
   leaves the Value generic.

   This class can extend either ChainHashTable or OpenHashTable.

                        HashTable<K,V>
                       /              \
                      /                \
     ChainHashTable<K,V>              OpenHashTable<K,V>
                                              |
                                              |
                                        StringHashTable<V>

   Use this class to experiment with different String hash functions
   and with the two collision policies.
*/
class StringHashTable<V> extends OpenHashTable<String, V> // try ChainHashTable
{
   public StringHashTable(int length)
   {
      super(length);
   }

   public int hashFunction(String key)
   {
      return key.length();  // This is a terrible hash function.
      //return key.charAt(0) + key.charAt(key.length()-1); // A little bit better.
      //return key.hashCode(); // Use Java's builtin hash function for strings.
   }
}


/**
   Test HashTable<K,V> using StringHashTable<V> and either
   ChainHashTable<String,V> or OpenHashTable<String,V>.
   Try changing the hash function in StringHashTable.
*/
public class StringHashTableDemo2
{
   public static void main(String[] args)
   {
      final StringHashTable<String> names = new StringHashTable<>(32);
      names.debug = false; // Don't show the rehashing steps.

      names.insert("Sue", "cat");
      names.insert("Harry", "cat");
      names.insert("Nina", "dog");
      names.insert("Susannah", "cat");
      names.insert("Ed", "dog");
      names.insert("Larry", "cat");
      names.insert("Eve", "cat");
      names.insert("Sarah", "dog");
      names.insert("Adam", "dog");
      names.insert("Tony", "cat");
      names.insert("Katherine", "cat");
      names.insert("Juliet", "dog");
      names.insert("Romeo", "cat");
      names.insert("Barry", "cat");

      System.out.println("Hash table.");
      System.out.println( names );

      names.delete("Harry");
      names.delete("Barry");
      names.delete("Larry");
      names.delete("Romeo");
      names.delete("Sarah");

      System.out.println("Hash table.");
      System.out.println( names );

      names.delete("Katherine");
      names.insert("Katherine", "dog");
      names.delete("Eve");
      names.insert("Eve", "dog");

      System.out.println("Hash table.");
      System.out.println( names );

      names.delete("Tony");
      names.delete("Juliet");
      names.insert("Tony", "dog");
      names.insert("Juliet", "cat");

      System.out.println("Hash table.");
      System.out.println( names );
   }
}
